<?php
/**
 * License Logic
 * @package     License
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class License_Logic
{
    /**
     * DAOオブジェクト
     * @var License_Dao
     */
    private $dao;
    /**
     * ライセンスタイプ名
     * @var array
     */
    static public $type_name_list = array(
        'ME1P' => '製品ライセンス',
        'ME1F' => '非商用個人ライセンス',
        'ME1D' => '開発ライセンス',
    );

    /**
     * コンストラクタ
     */
    public function __construct()
    {
        $this->dao = new License_Dao();
    }

    /**
     * ライセンスキーを削除してオブジェクトを返す。
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function select($_id, $class_name = null)
    {
        return $this->dao->select($_id, $class_name);
    }

    /**
     * ライセンスキーも含めてオブジェクトを返す。
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function select_license_key($_id, $class_name = null)
    {
        return $this->dao->select_license_key($_id, $class_name);
    }

    /**
     * ライセンス登録
     * @param License_Form $form
     * @return boolean
     */
    public function register_license(License_Form $form)
    {
        // admin以外のばあい
        if (!User_Role::require_admin(false)) {
            throw new Me_Exception(608);
        }

        if (!$this->validation($form)) {
            return false;
        }
        // ライセンスタイプ設定
        $pieces = explode('-', $form->license_id);
        $form->license_type = $pieces[0];

        if (empty($form->_id)) {
            // 新規追加
            $form->_id = 1;
            $this->dao->insert($form);
        } else {
            // 編集
            $form->_id = 1;
            $this->dao->update($form);
        }
        return true;
    }

    /**
     * 入力値チェック
     * @param License_Form $form
     * @return boolean
     */
    public function validation(License_Form $form)
    {
        // エラーメッセージを初期化
        $form->errors = array();

        // license_id
        // 最小文字数チェック
        if (strlen($form->license_id) < 1) {
            $form->errors['license_id'] = 'ライセンスIDを入力して下さい';
        } else if (strlen($form->license_id) > 20) {
            // 最大文字数チェック
            $form->errors['license_id'] = 'ライセンスIDは20文字以内で入力して下さい';
        } else if (!preg_match('/^[a-z0-9@._-]+$/i', $form->license_id)) {
            // 文字種チェック
            $form->errors['license_id'] = 'ライセンスIDは、英字、数字、ハイフン( - ),' .
                ' アンダースコア( _ ), ピリオド( . ), アットマーク( @ )のみで入力して下さい';
        }

        // license_key
        // 最小文字数チェック
        if (strlen($form->license_key) < 4) {
            $form->errors['license_key'] = 'ライセンスキーを入力して下さい';
        } else if (strlen($form->license_key) > 23) {
            //最大文字数チェック
            $form->errors['license_key'] = 'ライセンスキーは23文字以内で入力して下さい';
        } else if (!preg_match('/^[a-z0-9@._-]+$/i', $form->license_key)) {
            //文字種チェック
            $form->errors['license_key'] = 'ライセンスキーは、英字、数字ハイフン( - ),' .
                ' アンダースコア( _ ), ピリオド( . ), アットマーク( @ )のみで入力して下さい';
        }

        // ライセンスキー認証
        if (empty($form->errors) && !$this->authentication($form->license_id, $form->license_key)) {
            $form->errors['license_error'] = 'ライセンスIDとライセンスキーの認証に失敗しました。';
        }

        // エラー有無確認
        if (!empty($form->errors)) {
            $form->errors['MESSAGE'] = 'エラーが発生しました。';
            return false;
        }
        return true;
    }

    /**
     * ライセンスキー認証
     * @param string $license_id
     * @param string $license_key
     * @return boolean
     */
    public function authentication($license_id, $license_key)
    {
        if (!strlen($license_id) || !strlen($license_key)) {
            return false;
        }
        $id_hash = sha1($license_id);
        $sec_keys = array(
            'ME1P' => '58f71d5978d5cd11ad54edce48d68ddc5c3121eb',
            'ME1F' => 'ea8937fd5e4e418b70c73f64c781bb376ea52062',
            'ME1D' => '32793db5c14d2fdc28d56813c3b4fa1880046045',
        );
        $pieces = explode('-', $license_id);
        if (count($pieces) !== 2 || !isset($sec_keys[$pieces[0]])) {
            return false;
        }
        $sec = $sec_keys[$pieces[0]];
        $key_char = array(
            '1' => -15, '2' => -14, '3' => -13, '4' => -12, '5' => -11,
            '6' => -10, '7' => -9, '8' => -8, '9' => -7, 'A' => -6,
            'B' => -5, 'C' => -4, 'D' => -3, 'E' => -2, 'F' => -1,
            'G' => 0, 'H' => 1, 'J' => 2, 'K' => 3, 'L' => 4,
            'M' => 5, 'N' => 6, 'P' => 7, 'R' => 8, 'S' => 9,
            'T' => 10, 'U' => 11, 'V' => 12, 'W' => 13, 'X' => 14,
            'Y' => 15,
        );
        $license_key = str_replace('-', '', $license_key);

        $key = '';
        for ($i = 0; $i < 20; $i++) {
            $id_dec = hexdec($id_hash[$i * 2]);
            $diff = $key_char[$license_key[$i]];
            $key .= dechex($id_dec + $diff);
        }
        return $sec === sha1($key);
    }
}